/*
 * time_base.c
 *
 *  Created on: Jan 13, 2024
 *      Author: hussamaldean
 */


#include "stm32f7xx.h"
#include "time_base.h"
#include "lvgl.h"



#define	CTRL_ENABLE					(1U<<0) /*Enable SysTick Timer*/
#define CTRL_CLKSRC					(1U<<2) /*Clock source selection*/
#define CTRL_COUNTFLAG				(1U<<16) /*Count flag bit*/
#define CTRL_TICKINT				(1U<<1) /*Interrupt enable bit*/


volatile uint64_t mil; /*volatile variable to hold the ms counter*/




void Time_Base_Init(uint32_t  freq)
{
	/*Disable global interrupt*/
	__disable_irq();

	/*Set period to be 1ms*/
	SysTick->LOAD=(freq/1000)-1;

	/*Reset the systick value*/
	SysTick->VAL=0;

	/*Enable systick and select internal clk src*/
	SysTick->CTRL = CTRL_ENABLE | CTRL_CLKSRC ;

	/*Enable systick interrupt*/
	SysTick->CTRL  |= CTRL_TICKINT;

	/*Enable global interrupt*/
	__enable_irq();
}

uint64_t Tick_Get(void)
{
	__disable_irq(); /*Disable global interrupt*/

	uint64_t ml=mil; /*Get the current millis values and store in ml*/

	__enable_irq(); /*Enable global interrupt*/

	return ml;		/*Return the stored value*/
}


/*Spin lock the CPU to delay*/
void delay(uint32_t time)
{

	uint64_t start=Tick_Get();
	while((Tick_Get() - start) < (time+1));
}

/*Interrupt handler of SysTick*/
void SysTick_Handler(void)
{
	/*Increment the counter with every interrupt*/
	mil++;

	/*Call LvGL tick function*/
	lv_tick_inc(1);
}
